@@ -14,6 +14,9 @@
 #import "DaliClockView.h"
 
 @interface AppController : NSObject
+# if MAC_OS_X_VERSION_MAX_ALLOWED > MAC_OS_X_VERSION_10_5
+   <NSWindowDelegate>
+# endif
 {
   NSWindow *window;
   DaliClockView *view;
