@@ -1,5 +1,5 @@
 /* xdaliclock - a melting digital clock
- * Copyright (c) 1991-2010 Jamie Zawinski <jwz@jwz.org>
+ * Copyright (c) 1991-2012 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -263,6 +263,7 @@ static NSUserDefaultsController *staticUserDefaultsController = 0;
   ogl_ctx = [[NSOpenGLContext alloc] 
               initWithFormat:pixfmt
                 shareContext:nil];
+  check_gl_error ("init ctx");
 
 #  if MAC_OS_X_VERSION_MAX_ALLOWED <= MAC_OS_X_VERSION_10_4
   long svarg;
@@ -273,16 +274,21 @@ static NSUserDefaultsController *staticUserDefaultsController = 0;
   // Sync refreshes to the vertical blanking interval
   svarg = 1;
   [ogl_ctx setValues:&svarg forParameter:NSOpenGLCPSwapInterval];
+  check_gl_error ("init swapinterval");
 
   // Make window transparency possible
   svarg = NO;
   [ogl_ctx setValues:&svarg forParameter:NSOpenGLCPSurfaceOpacity];
+  check_gl_error ("init opacity");
 
 
   [ogl_ctx makeCurrentContext];
+  check_gl_error ("init makeCurrentContext");
 
   // Clear frame buffer ASAP, else there are bits left over from other apps.
-  glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
+  // #### On 10.7, this causes GL_INVALID_FRAMEBUFFER_OPERATION.
+  //  glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
+  //  check_gl_error ("init clear");
 
   // Enable multi-threading, if possible.  This runs most OpenGL commands
   // and GPU management on a second CPU.
@@ -295,6 +301,7 @@ static NSUserDefaultsController *staticUserDefaultsController = 0;
     if (err != kCGLNoError) {
       NSLog (@"enabling multi-threaded OpenGL failed: %d", err);
     }
+    check_gl_error ("init threading");
   }
 
   {
@@ -304,6 +311,15 @@ static NSUserDefaultsController *staticUserDefaultsController = 0;
       glDrawBuffer (GL_BACK);
     else
       glDrawBuffer (GL_FRONT);
+
+    /* WTF?  Sometimes we get "invalid operation" when doing GL_FRONT,
+       which means the front buffer doesn't exist!  But that's a lie,
+       because on ignoring the error, everything works.  So, just flush
+       errors here.
+     */
+    // check_gl_error ("init drawBuffer");
+    while (glGetError() != GL_NO_ERROR)
+      ;
   }
 
   check_gl_error ("init");
@@ -369,15 +385,37 @@ static NSUserDefaultsController *staticUserDefaultsController = 0;
 
 # else  /* USE_IPHONE */
   [self didRotate:nil];
-# endif /* !USE_IPHONE */
+# endif /* USE_IPHONE */
 
   int ow = config.width;
   int oh = config.height;
 
-   config.width  = newSize.width * 2;   // use the next-larger bitmap
-   config.height = newSize.height * 2;
-//  config.width = 1280;    // always use the biggest font image
-//  config.height = 1024;
+# if defined(USE_IPHONE) && defined(__IPHONE_4_0)
+  /* Make sure we are dealing with real pixels, not "virtual" pixels. */
+  if ([self respondsToSelector:@selector(contentScaleFactor)]) {
+    double s = [self contentScaleFactor];
+    newSize.width  *= s;
+    newSize.height *= s;
+  }
+# endif /* __IPHONE_4_0 */
+
+  float sscale = 2.5;   // use the next-larger bitmap
+# ifndef USE_IPHONE
+  config.width  = newSize.width  * sscale;
+  config.height = newSize.height * sscale;
+# else  /* USE_IPHONE */
+  switch (orientation) {
+  case UIInterfaceOrientationLandscapeRight:
+  case UIInterfaceOrientationLandscapeLeft:
+    config.width  = newSize.height * sscale;
+    config.height = newSize.width  * sscale;
+    break;
+  default:
+    config.width  = newSize.width  * sscale;
+    config.height = newSize.height * sscale;
+    break;
+  }
+# endif /* USE_IPHONE */
 
   render_bitmap_size (&config, 
                       &config.width, &config.height,
@@ -398,13 +436,21 @@ static NSUserDefaultsController *staticUserDefaultsController = 0;
 
 # ifdef USE_IPHONE
   if (rotation_ratio < 0) {		// Not currently doing a rotation
-    rot_to.width = config.width;
-    rot_to.height = config.height;
-    rot_from = rot_to;
+    switch (orientation) {
+    case UIInterfaceOrientationLandscapeLeft:
+    case UIInterfaceOrientationLandscapeRight:
+      rot_to.width  = newSize.height;
+      rot_to.height = newSize.width;
+      break;
+    default:
+      rot_to.width  = newSize.width;
+      rot_to.height = newSize.height;
+      break;
+    }
+    rot_from   = rot_to;
     angle_from = angle_to;
   }
 # endif /* USE_IPHONE */
-
 }
 
 
@@ -434,6 +480,15 @@ static NSUserDefaultsController *staticUserDefaultsController = 0;
 {	
   UIDeviceOrientation current = [[UIDevice currentDevice] orientation];
 
+  if (current == UIDeviceOrientationUnknown) {
+    /* On the iPad (but not iPhone 3GS, or the simulator) sometimes we get
+       an orientation change event with an unknown orientation.  Those seem
+       to always be immediately followed by another orientation change with
+       a *real* orientation change, so let's try just ignoring those bogus
+       ones and hoping that the real one comes in shortly... */
+    return;
+  }
+
   if (rotation_ratio >= 0) return;	// in the midst of rotation animation
   if (orientation == current) return;	// no change
 
@@ -454,27 +509,38 @@ static NSUserDefaultsController *staticUserDefaultsController = 0;
   default:					 angle_to = 0;   break;
   }
 
+  NSRect ff = [self frame];
+
+# if defined(__IPHONE_4_0)
+  /* Make sure we are dealing with real pixels, not "virtual" pixels. */
+  if ([self respondsToSelector:@selector(contentScaleFactor)]) {
+    double s = [self contentScaleFactor];
+    ff.size.width  *= s;
+    ff.size.height *= s;
+  }
+# endif /* __IPHONE_4_0 */
+
   switch (orientation) {
   case UIInterfaceOrientationLandscapeLeft:	// from landscape
   case UIInterfaceOrientationLandscapeRight:
-    rot_from.width  = [self frame].size.height;
-    rot_from.height = [self frame].size.width;
+    rot_from.width  = ff.size.height;
+    rot_from.height = ff.size.width;
     break;
   default:					// from portrait
-    rot_from.width  = [self frame].size.width;
-    rot_from.height = [self frame].size.height;
+    rot_from.width  = ff.size.width;
+    rot_from.height = ff.size.height;
     break;
   }
 
   switch (new_orientation) {
   case UIInterfaceOrientationLandscapeLeft:	// to landscape
   case UIInterfaceOrientationLandscapeRight:
-    rot_to.width  = [self frame].size.height;
-    rot_to.height = [self frame].size.width;
+    rot_to.width  = ff.size.height;
+    rot_to.height = ff.size.width;
     break;
   default:					// to portrait
-    rot_to.width  = [self frame].size.width;
-    rot_to.height = [self frame].size.height;
+    rot_to.width  = ff.size.width;
+    rot_to.height = ff.size.height;
     break;
   }
 }
@@ -819,6 +885,7 @@ FAIL:
 }
 
 
+#if 0
 #ifndef USE_IPHONE
 - (void)viewDidMoveToWindow
 {
@@ -826,6 +893,12 @@ FAIL:
   // But only do this if our creator has said that it's ok for us to diddle
   // the background color of the window we are running on.
   //
+  // #### For some reason we are now getting "invalid context 0" errors
+  //      inside AppController:[window setContentView:view] when doing
+  //      this here.  So, this code was moved to inside of colorTick,
+  //      which means the window flickers at startup (initially has a
+  //      solid background color, then turns transparent.)  Bleh.
+  //
   if (ownWindow) {
     [[self window] setBackgroundColor:[NSColor clearColor]];
     [[self window] setOpaque:NO];
@@ -834,6 +907,7 @@ FAIL:
   }
 }
 #endif /* USE_IPHONE */
+#endif /* 0 */
 
 
 /* The big kahuna refresh method.  Draw the current contents of the bitmap
@@ -851,6 +925,16 @@ FAIL:
 
 # ifdef USE_IPHONE
 
+#  if defined(__IPHONE_4_0)
+  /* Make sure we are dealing with real pixels, not "virtual" pixels. */
+  if ([self respondsToSelector:@selector(contentScaleFactor)]) {
+    double s = [self contentScaleFactor];
+    framerect.size.width  *= s;
+    framerect.size.height *= s;
+    framerect2 = framerect;
+  }
+#  endif /* __IPHONE_4_0 */
+
   GLfloat angle = 0;
 
   // No rotation animation if this is the first time.
@@ -1131,6 +1215,18 @@ FAIL:
     colorTimer = 0;
   }
 
+# ifndef USE_IPHONE
+  // See comment in viewDidMoveToWindow -- doing this there caused errors.
+  // 
+  if ([self window] && ownWindow) {
+    [[self window] setBackgroundColor:[NSColor clearColor]];
+    [[self window] setOpaque:NO];
+    [[NSColor clearColor] set];
+    NSRectFill ([self frame]);
+    ownWindow = 0; // only do it once
+  }
+# endif /* !USE_IPHONE */
+
   if (config.max_cps <= 0) return;   // cycling is turned off, do nothing
 
   if ([self window]		     // do nothing if no window yet
